using UnityEngine;

public class ItemActionMagicWand1 : ItemAction
{
    // --------------------------------------------------------------------------------------------------
    // --------------------------------------------------------------------------------------------------
    // Magic Wand v1
    // -------------
    // by Matite
    // 6th November 2015
    //
    // This script is for use with 7 Days to Die (A12.5) and SD2DX (v4.2). When holding the magic wand 
    // item the player is able to look at a location and left click the mouse button to instantly 
    // teleport there. If the player holds down the SHIFT key when left clicking a line will also be 
    // drawn between their start and end positions.
    // --------------------------------------------------------------------------------------------------
    // --------------------------------------------------------------------------------------------------

    /// <summary>
    /// Stores a material (red) that is used for the LineRenderer line (when holding down SHIFT and left clicking)
    /// </summary>
    private Material matLine;

    /// <summary>
    /// Stores a LineRenderer used to draw the teleport line (when holding down SHIFT and left clicking)
    /// </summary>
    private LineRenderer lrLine;

    // --------------------------------------------------------------------------------------------------

    /// <summary>
    /// Magic Wand ItemActionData class
    /// </summary>
    public class InventoryDataMagicWand1 : ItemActionData
    {
        public bool bReleased;

        public InventoryDataMagicWand1(ItemInventoryData _inventoryData, int _indexInEntityOfAction)
            : base(_inventoryData, _indexInEntityOfAction)
        {

        }
    }


    public override ItemActionData CreateModifierData(ItemInventoryData _invData, int _indexInEntityOfAction)
    {
        return new InventoryDataMagicWand1(_invData, _indexInEntityOfAction);
    }


    /// <summary>
    /// Called when the Item is used (i.e. left click in this case as the script is attached to "Action0")
    /// </summary>
    /// <param name="_actionData"></param>
    /// <param name="_bReleased"></param>
    public override void ExecuteAction(ItemActionData _actionData, bool _bReleased)
    {
        InventoryDataMagicWand1 TheMagicWand = (InventoryDataMagicWand1)_actionData;

        // Check holding entity released state
        if (_bReleased)
        {
            TheMagicWand.bReleased = true;
        }
        else if(TheMagicWand.bReleased)
        {
            TheMagicWand.bReleased = false;

            // Set the holding entity use option to move the players right arm
            _actionData.invData.holdingEntity.RightArmAnimationUse = true;

            // Call the function to execute the custom code
            DoExecuteAction();
        }
    }


    /// <summary>
    /// Outputs text to the chat text area ingame
    /// </summary>
    /// <param name="str">The string to be displayed</param>
    private void DisplayChatAreaText(string str)
    {
        // Check if the game instance is not null
        if (GameManager.Instance != null)
        {
            // Display the string in the chat text area
            GameManager.Instance.GameMessageClient(str, "");
        }
    }
    

    /// <summary>
    /// Custom function called when the item is being held and the left mouse button is clicked
    /// </summary>
	private void DoExecuteAction()
    {
        // Debug       
        //DisplayChatAreaText("DoExecuteAction function (left mouse button clicked)");

        // Get a reference to the player
        EntityPlayerLocal player = GameManager.Instance.World.GetLocalPlayer();

        // Get a vector position for the players camera
        Vector3 vec3CameraPos = player.cameraTransform.position;
        //DisplayChatAreaText("Camera Pos: X = " + vec3CameraPos.x + "   Y = " + vec3CameraPos.y + "   Z = " + vec3CameraPos.z);

        // Get the position half a unit in front of the camera (need to ray cast away from the player so the ray does not hit their head or arm etc)
        Vector3 vec3CameraDirection = player.cameraTransform.forward;
        Vector3 vec3CameraPosOffset = vec3CameraPos + vec3CameraDirection * 0.5f;
        //DisplayChatAreaText("Camera Pos with Offset: X = " + vec3CameraPosOffset.x + "   Y = " + vec3CameraPosOffset.y + "   Z = " + vec3CameraPosOffset.z);

        // Get the forward vector for the players camera
        Vector3 fwd = player.cameraTransform.TransformDirection(Vector3.forward);
        //DisplayChatAreaText("Camera Pos Forward Vector: X = " + fwd.x + "   Y = " + fwd.y + "   Z = " + fwd.z);

        // Create a ray cast hit variable
        RaycastHit hitInfo;

        // Perform a ray cast starting at the players camera position and moving forward 500 units
        if (Physics.Raycast(vec3CameraPosOffset, fwd, out hitInfo, 500.0f, -1))
        {
            // Debug
            //DisplayChatAreaText("Collider Name: " + hitInfo.collider.name + "(X = " + hitInfo.transform.position.x + "   Y = " + hitInfo.transform.position.y + "   Z = " + hitInfo.transform.position.z + ")");

            // Check if the left or right shift key is being pressed (when the shift key is held down we draw a red line between the teleport start and end positions)
            if (Input.GetKey(KeyCode.LeftShift) || Input.GetKey(KeyCode.RightShift))
            {
                // Debug
                //DisplayChatAreaText("DoExecuteAction function (left mouse button and shift key)");

                // Handle errors gracefully (instead of popping up the console with red text)
                try
                {
                    // Check if we do not yet have a material created (used on the LineRenderer)
                    if (!matLine)
                    {
                        // Create a new material with red color
                        matLine = new Material(Shader.Find("Specular"));
                        matLine.color = Color.red;

                        // Debug
                        //DisplayChatAreaText("Created material (for LineRenderer)");
                    }
                }
                catch (System.Exception ex)
                {
                    // Display an error message in the chat text area
                    DisplayChatAreaText("Exception caught while attempting to create a new Material (matLine): " + ex.Message);
                }

                // Handle errors gracefully (instead of popping up the console with red text)
                try
                {
                    // Check if we do not yet have a LineRenderer component created
                    if (!lrLine)
                    {
                        // Create a new LineRenderer
                        lrLine = new GameObject("Line0").AddComponent<LineRenderer>();

                        // Assign the material (red) to the LineRenderer
                        lrLine.material = matLine;

                        // Set the number of points for the LineRenderer line (2... start point and end point)
                        lrLine.SetVertexCount(2);

                        // Set the width of the LineRenderer line
                        lrLine.SetWidth(0.15f, 0.01f);

                        // Render line to the world origin and not to the object's position
                        lrLine.useWorldSpace = true;

                        // Debug
                        //DisplayChatAreaText("Created LineRenderer");
                    }
                }
                catch (System.Exception ex)
                {
                    // Display an error message in the chat text area
                    DisplayChatAreaText("Exception caught while attempting to create a new LineRenderer (lrLine): " + ex.Message);
                }

                // Set LineRenderer line start (in front of player) and end (teleport location target) positions
                lrLine.SetPosition(0, vec3CameraPosOffset);
                lrLine.SetPosition(1, hitInfo.point);
            }

            // Teleport the player to the new position
            player.SetPosition(new Vector3(hitInfo.point.x, hitInfo.point.y, hitInfo.point.z));

            // Display a message in the chat text area
            DisplayChatAreaText("Teleported to: X = " + hitInfo.point.x + "   Y = " + hitInfo.point.y + "   Z = " + hitInfo.point.z);
        }
        else
        {
            // Display a message in the chat text area
            DisplayChatAreaText("Sorry, teleport is not possible (no collider found).");
        }
        
    }


	/// <summary>
    /// Called when the player starts holding the item
    /// </summary>
    /// <param name="_actionData"></param>
    public override void StartHolding(ItemActionData _actionData)
    {
        InventoryDataMagicWand1 TheMagicWand = (InventoryDataMagicWand1)_actionData;
        TheMagicWand.bReleased = true;
    }

    
    /// <summary>
    /// Called when the player stops holding the item
    /// </summary>
    /// <param name="_data"></param>
    public override void StopHolding(ItemActionData _data)
    {
        InventoryDataMagicWand1 TheMagicWand = (InventoryDataMagicWand1)_data;
    }

}